@@ -83,7 +83,9 @@ it will happen soon. *)
     mutable op_file_to_option : ('a -> (string * option_value) list);
     mutable op_file_cancel : ('a -> unit);
     mutable op_file_pause : ('a -> unit);
+    mutable op_file_queue : ('a -> unit);
     mutable op_file_resume : ('a -> unit);
+    mutable op_file_download_order : ('a -> swarming_strategy option -> swarming_strategy option);
     mutable op_file_info : ('a -> GuiTypes.file_info);
     mutable op_file_set_format : ('a -> CommonTypes.format -> unit);
     mutable op_file_check : ('a -> unit);
@@ -91,7 +93,6 @@ it will happen soon. *)
     mutable op_file_all_sources : ('a -> client list);
     mutable op_file_active_sources : ('a -> client list);
     mutable op_file_comment : ('a -> string);
-    mutable op_file_set_priority : ('a -> int -> unit);
     mutable op_file_print : ('a -> CommonTypes.ui_conn -> unit);
     mutable op_file_print_sources : ('a -> CommonTypes.ui_conn -> unit);
     mutable op_file_files : ('a -> 'a file_impl -> file list);    
@@ -140,8 +141,8 @@ let dummy_file_impl = {
     impl_file_last_seen = 0;
     impl_file_comment = "";
     impl_file_probable_name = None;
-    impl_file_owner = admin_user;
-    impl_file_group = Some system_user_default_group;
+    impl_file_owner = admin_user ();
+    impl_file_group = Some (admin_group ());
   }
 
 let dummy_file = as_file dummy_file_impl
@@ -245,11 +246,19 @@ let file_pause (file : file) user =
       file.impl_file_ops.op_file_pause file.impl_file_val
   | _ -> ()
 
+let file_queue (file : file) =
+  let file = as_file_impl file in
+  match file.impl_file_state with
+  | FileDownloading ->
+      update_file_state file FileQueued;
+      file.impl_file_ops.op_file_queue file.impl_file_val
+  | _ -> ()
+
 let file_resume (file : file) user =
   if user2_allow_file_admin file user then
   let file = as_file_impl file in
   match file.impl_file_state with
-  | FilePaused | FileAborted _ ->
+  | FileQueued | FilePaused | FileAborted _ ->
       update_file_state file FileDownloading;
       file.impl_file_ops.op_file_resume file.impl_file_val
   | _ -> ()
@@ -274,14 +283,28 @@ let user2_filter_files files gui_user =
     (fun file -> user2_can_view_file gui_user (file_owner file) (file_group file)) files in
   newlist
 
+let file_state file =
+  let impl = as_file_impl file in
+  impl.impl_file_state
+
 let user2_num_user_dls user =
+  let file_is_in_download_queue file =
+    match file_state file with
+    | FileNew | FileDownloading | FileQueued | FilePaused | FileDownloaded -> true
+    | FileShared | FileCancelled | FileAborted _ -> false
+  in
   let n = ref 0 in
-  H.iter (fun f -> if file_owner f = user then incr n) files_by_num;
+  H.iter (fun f -> if file_owner f = user && file_is_in_download_queue f then incr n) files_by_num;
   !n
 
 let user2_num_group_dls group =
+  let file_is_in_download_queue file =
+    match file_state file with
+    | FileNew | FileDownloading | FileQueued | FilePaused | FileDownloaded -> true
+    | FileShared | FileCancelled | FileAborted _ -> false
+  in
   let n = ref 0 in
-  H.iter (fun f -> if file_group f = Some group then incr n) files_by_num;
+  H.iter (fun f -> if file_group f = Some group && file_is_in_download_queue f then incr n) files_by_num;
   !n
 
 let set_file_state file state =
@@ -374,12 +397,12 @@ let default_file_print_sources file o =
   if List.length !allsources > 0 then begin
 
     html_mods_table_header buf "sourcesTable" "sources al" [
-        ( "1", "srh br ac", "Client number", "Num" ) ;
-        ( "0", "srh br", "Client Name", "Name" ) ;
-        ( "0", "srh br", "IP address", "IP address" ) ;
-        ( "0", "srh br", "Client software", "CS" ) ;
-        ( "1", "srh ar", "Total UL bytes to this client for all files", "UL" ) ;
-        ( "1", "srh ar br", "Total DL bytes from this client for all files", "DL" ) ; ];
+        ( Num, "srh br ac", "Client number", "Num" ) ;
+        ( Str, "srh br", "Client Name", "Name" ) ;
+        ( Str, "srh br", "IP address", "IP address" ) ;
+        ( Str, "srh br", "Client software", "CS" ) ;
+        ( Num, "srh ar", "Total UL bytes to this client for all files", "UL" ) ;
+        ( Num, "srh ar br", "Total DL bytes from this client for all files", "DL" ) ; ];
 
     html_mods_cntr_init ();
 
@@ -436,10 +459,6 @@ let file_find num =
     dummy_file_impl   with impl_file_num = num
   })
 
-let file_state file =
-  let impl = as_file_impl file in
-  impl.impl_file_state
-
 let file_add_source (file : file) c =
   client_must_update c;
   CommonEvent.add_event (File_add_source_event (file,c))
@@ -522,7 +541,6 @@ let set_file_priority file p =
   let impl = as_file_impl file in
   if impl.impl_file_priority <> p then begin
       impl.impl_file_priority <- p;
-      impl.impl_file_ops.op_file_set_priority  impl.impl_file_val p;
       file_must_update file
     end
 
@@ -558,6 +576,19 @@ let file_preview (file : file) =
       [| Filename2.basename !!previewer; file_disk_name file; file_best_name file |]
       Unix.stdin Unix.stdout Unix.stderr)
 
+let file_download_order (file : file) strategy =
+  let file = as_file_impl file in
+  file.impl_file_ops.op_file_download_order file.impl_file_val strategy
+
+let file_print_download_order (file : file) =
+  let file = as_file_impl file in
+  match file.impl_file_ops.op_file_download_order file.impl_file_val None with
+  | None -> "unknown"
+  | Some strategy ->
+      match strategy with
+      | LinearStrategy -> "linear"
+      | AdvancedStrategy -> "random"
+
 (*************************************************************************)
 (*                                                                       *)
 (*                         file_downloaders                              *)
@@ -669,12 +700,29 @@ let file_print file o =
   let buf = o.conn_buf in
   let srcs = file_all_sources file in
 
+  let chunks_counts chunks = 
+	  let tc = VerificationBitmap.length chunks in
+	  let c0 = ref 0 in
+	  let c1 = ref 0 in
+	  let c2 = ref 0 in
+	  let c3 = ref 0 in
+
+	  VerificationBitmap.iteri (fun _ c ->
+            match c with
+            | VerificationBitmap.State_missing -> incr c0
+            | VerificationBitmap.State_partial -> incr c1
+            | VerificationBitmap.State_complete -> incr c2
+            | VerificationBitmap.State_verified -> incr c3
+	  ) chunks;
+  	Printf.sprintf "%d = %d + %d + %d + %d" tc !c0 !c1 !c2 !c3
+  in
+
   if use_html_mods o then begin
 
       html_mods_cntr_init ();
       html_mods_table_header buf "sourcesInfo" "sourcesInfo" [
-        ( "0", "srh br", "File Info", "Info" ) ;
-        ( "0", "srh", "Value", "Value" ) ];
+        ( Str, "srh br", "File Info", "Info" ) ;
+        ( Str, "srh", "Value", "Value" ) ];
 
       Printf.bprintf buf "\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
       html_mods_td buf [
@@ -693,7 +741,7 @@ let file_print file o =
         ("", "sr", Printf.sprintf "%d" (file_priority file)) ];
 
       Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
-      if user2_allow_file_admin file o.conn_user.ui_user then
+      if user2_is_admin o.conn_user.ui_user then
         let optionlist = ref "" in
         user2_users_iter (fun user ->
 	  if user <> (file_owner file) then
@@ -725,7 +773,7 @@ parent.fstatus.location.href='submit?q=chown+'+v+'+%d';
 
       Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
       if user2_allow_file_admin file o.conn_user.ui_user &&
-         o.conn_user.ui_user.user_groups <> [] then
+         (file_owner file).user_groups <> [] then begin
         let optionlist =
           if (file_group file) = None then
             ref ""
@@ -756,6 +804,7 @@ parent.fstatus.location.href='submit?q=chgrp+'+v+'+%d';
    ^ Printf.sprintf "\\<option value=\\\"%s\\\" selected\\>%s\\</option\\>\n" (user2_print_group (file_group file)) (user2_print_group (file_group file))
    ^ !optionlist ^ "\\</select\\>\\</td\\>\\</form\\>\\</tr\\>\\</table\\>"
   ) ];
+        end
 
       else
         html_mods_td buf [("File group", "sr br", "Group");
@@ -763,6 +812,11 @@ parent.fstatus.location.href='submit?q=chgrp+'+v+'+%d';
           Some group -> Printf.sprintf "%s" group.group_name
         | None -> "None"))];
 
+      Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
+      html_mods_td buf [
+        ("Download strategy", "sr br", "DL strategy");
+        ("", "sr", file_print_download_order file) ];
+
       Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
       html_mods_td buf [
         ("Number of file sources", "sr br", "Sources");
@@ -770,35 +824,28 @@ parent.fstatus.location.href='submit?q=chgrp+'+v+'+%d';
 
       Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
 
-      (match info.G.file_chunks with 
+      begin match info.G.file_chunks with 
       | None -> ()
       | Some chunks ->
-	  let tt = "0=Missing, 1=Partial, 2=Complete, 3=Verified" in
-	  let tc = VerificationBitmap.length chunks in
-	  let c0 = ref 0 in
-	  let c1 = ref 0 in
-	  let c2 = ref 0 in
-	  let c3 = ref 0 in   
-
-	  VerificationBitmap.iteri (fun _ c ->
-            match c with
-            | VerificationBitmap.State_missing -> incr c0
-            | VerificationBitmap.State_partial -> incr c1
-            | VerificationBitmap.State_complete -> incr c2
-            | VerificationBitmap.State_verified -> incr c3
-	  ) chunks;
-      
-	  let header = Printf.sprintf "%d (%d+%d+%d+%d): " tc !c0 !c1 !c2 !c3 in
+        let tt = "Total = Missing + Partial + Complete + Verified" in
+        let summary = chunks_counts chunks in
 
 	  html_mods_td buf [
             (tt, "sr br", "Chunks");
             (tt, "sr", 
-            header ^ if !!html_vd_chunk_graph then
+            summary ^ if !!html_vd_chunk_graph then
               colored_chunks chunks
             else
               VerificationBitmap.to_string chunks
             ) ]
-      );
+      end;
+
+      Printf.bprintf buf "\\</tr\\>\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
+      html_mods_td buf [
+        ("Chunk size", "sr br", "Chunk size");
+        ("", "sr", (match info.G.file_chunk_size with
+          Some v -> String.concat " " (List.map (fun v -> Printf.sprintf "%Ld" v) v)
+        | None -> "unknown"))];
 
       (match file_magic file with
         Some magic ->
@@ -830,10 +877,9 @@ parent.fstatus.location.href='submit?q=chgrp+'+v+'+%d';
 	(match file_group file with
 	   Some group -> Printf.sprintf "%s" group.group_name
 	 | None -> "private");
-      Printf.bprintf buf "Chunks: [%-s]\n"
 	(match info.G.file_chunks with
-	| None -> ""
-	| Some chunks -> VerificationBitmap.to_string chunks);
+      | None -> () 
+      | Some chunks -> Printf.bprintf buf "Chunks: %s\n" (chunks_counts chunks));
       (match impl.impl_file_probable_name with
           None -> ()
         | Some filename ->
@@ -988,7 +1034,9 @@ let new_file_ops network =
       op_file_info = (fun _ -> fni network "file_info");
       op_file_files = (fun _ impl -> [as_file impl]);
       op_file_pause = (fun _ -> ni_ok network "file_pause");
+      op_file_queue = (fun _ -> ni_ok network "file_queue");
       op_file_resume = (fun _ -> ni_ok network "file_resume");
+      op_file_download_order = (fun _ _ -> None);
 (*      op_file_disk_name = (fun _ -> fni network "file_disk_name"); *)
       op_file_check = (fun _ -> ni_ok network "file_check");
       op_file_recover = (fun _ -> ni_ok network "file_recover");
@@ -996,7 +1044,6 @@ let new_file_ops network =
       op_file_all_sources = (fun _ -> fni network "file_all_sources");
       op_file_active_sources = (fun _ -> fni network "file_active_sources");
       op_file_comment = (fun _ -> ni_ok network "file_comment"; "");
-      op_file_set_priority = (fun _ _ -> ni_ok network "file_set_priority");
       op_file_print = (fun _ _ -> ni_ok network "file_print_html");
       op_file_print_sources = (fun _ _ -> fni network "file_print_sources");
       op_file_debug = (fun _ -> "");
@@ -1027,8 +1074,12 @@ let check_file_implementations () =
         lprintf_nl "op_file_cancel";
       if c.op_file_pause == cc.op_file_pause then
         lprintf_nl "op_file_pause";
+      if c.op_file_queue == cc.op_file_queue then
+        lprintf_nl "op_file_queue";
       if c.op_file_resume == cc.op_file_resume then
         lprintf_nl "op_file_resume";
+      if c.op_file_download_order == cc.op_file_download_order then
+        lprintf_nl "op_file_download_order";
 (*      if c.op_file_disk_name == cc.op_file_disk_name then
         lprintf_nl "op_file_disk_name"; *)
       if c.op_file_check == cc.op_file_check then
@@ -1201,6 +1252,7 @@ let impl_file_info impl =
     T.file_active_sources = 0;
     T.file_sources = None;
     T.file_chunks = None;
+    T.file_chunk_size = None;
     T.file_availability = [];
     T.file_format = FormatNotComputed 0;
     T.file_chunks_age = [||];
@@ -1217,3 +1269,13 @@ let lprintf_file_nl file fmt =
   lprintf_nl2 ("[" ^ (file_network file).network_shortname ^
 	       "] [file_num " ^ (string_of_int (file_num file)) ^ "]" ^
 	       "[temp " ^ (file_disk_name file) ^ "]") fmt
+
+let concat_file dir filename =
+  let fs = Unix32.filesystem dir in
+  let namemax =
+    match Unix32.fnamelen dir with
+    | None -> 0
+    | Some v -> v
+  in
+  Filename.concat dir (Filename2.filesystem_compliant filename fs namemax)
+
