@@ -108,10 +108,12 @@ let version () =
 in an option, and then change the option accordingly. ?> *)
 let find_other_port = ref false
 
+let upnp_port_forwarding () = !!upnp_port_forwarding && Autoconf.upnp_natpmp
+
 let shorten str limit =
   (* TODO: we should change all strings to utf8 when
      they come into the core instead. *)
-  let name = Charset.to_utf8 (* String.escaped *) str in
+  let name = Charset.Locale.to_utf8 (* String.escaped *) str in
   let slen = String.length str in
   let len =
     try
@@ -246,8 +248,6 @@ let udp_write_controler = UdpSocket.new_bandwidth_controler upload_control
 
 let udp_read_controler = UdpSocket.new_bandwidth_controler download_control
 
-let gui_server_sock = ref (None : TcpServerSocket.t option)
-
 let pid = Unix.getpid ()
 
 let do_at_exit f =
@@ -447,12 +447,12 @@ let chat_message_fifo = (Fifo.create () : (int * string * int * string * string)
 let log_chat_message i num n s =
   Fifo.put chat_message_fifo (last_time(),i,num,n,s);
   (try
-    Unix2.tryopen_write_gen !messages_log [Open_creat; Open_wronly; Open_append] 
+    Unix2.tryopen_write_gen messages_log [Open_creat; Open_wronly; Open_append] 
       0o600 (fun oc ->
 	Printf.fprintf oc "%s: %s (%s): %s\n" (Date.simple (BasicSocket.date_of_int (last_time ()))) n i s)
   with e ->
     lprintf_nl "[ERROR] Exception %s while trying to log message to %s"
-      (Printexc2.to_string e) !messages_log);
+      (Printexc2.to_string e) messages_log);
 
   while (Fifo.length chat_message_fifo) > !!html_mods_max_messages do
     ignore(Fifo.take chat_message_fifo)
@@ -684,29 +684,6 @@ let partial_chunk c =
   | VerificationBitmap.State_complete | VerificationBitmap.State_verified -> 
       false
 
-let parse_magnet url =
-  let url = Url.of_string url in
-  if url.Url.short_file = "magnet:" then
-    let uids = ref [] in
-    let name = ref "" in
-    List.iter (fun (value, arg) ->
-        if String2.starts_with value "xt" then
-          uids := Uid.expand (Uid.of_string arg :: !uids)
-        else
-        if String2.starts_with value "dn" then
-          name := Url.decode arg
-        else
-        if arg = "" then
-(* This is an error in the magnet, where a & has been kept instead of being
-  url-encoded *)
-          name := Printf.sprintf "%s&%s" !name value
-        else
-          lprintf_nl "MAGNET: unused field %s = %s"
-            value arg
-    ) url.Url.args;
-    !name, !uids
-  else raise Not_found
-
     (*
 module CanBeCompressed = struct
 
@@ -859,7 +836,7 @@ let discover_ip force =
       if !verbose then lprintf_nl "started IP discovery";
       let module H = Http_client in
       let r = { H.basic_request with
-        H.req_url = Url.of_string "http://ip.discoveryvip.com/ip.asp";
+        H.req_url = Url.of_string "http://whatismyip.org/";
         H.req_proxy = !CommonOptions.http_proxy;
         H.req_max_retry = 10;
         H.req_user_agent = get_user_agent () }
@@ -870,16 +847,13 @@ let discover_ip force =
           try
             while true do
               let line = input_line cin in
-              let search_string = "Your Ip address is  " in
                 try
-                  if Str.string_match (Str.regexp ("^" ^ search_string)) line 0 then
-                    begin
-                      set_client_ip =:=
-                        Ip.of_string (String.sub line (String.length search_string)
-                          ((String.length line) - (String.length search_string)));
-                      last_high_id := !!set_client_ip;
-                      if !verbose then lprintf_nl "discovered IP %s" (Ip.to_string !!set_client_ip)
-                    end
+                  let ip = Ip.of_string line in
+                  begin
+                    set_client_ip =:= ip;
+                    last_high_id := !!set_client_ip;
+                    if !verbose then lprintf_nl "discovered IP %s" (Ip.to_string !!set_client_ip)
+                  end
                 with e -> lprintf_nl "IP discovery parse error: %s" (Printexc2.to_string e)
             done
           with End_of_file -> ())
