@@ -131,11 +131,11 @@ let windows_sleep seconds =
 let min_reserved_fds = 50
 let min_connections = 50
 
-let _ =
+let () =
   lprintf_nl "Starting MLDonkey %s ... " Autoconf.current_version;
   let ulof_old = Unix2.c_getdtablesize () in
   lprintf_nl "Language %s, locale %s, ulimit for open files %d"
-    Charset.default_language Charset.locstr ulof_old;
+    Charset.Locale.default_language Charset.Locale.locale_string ulof_old;
 
   let nofile = Unix2.ml_getrlimit Unix2.RLIMIT_NOFILE in
     if nofile.Unix2.rlim_max > 0 && nofile.Unix2.rlim_max > nofile.Unix2.rlim_cur then
@@ -208,10 +208,16 @@ let _ =
 
   (* Charset conversion self-test *)
   let filename = "abcdefghijklmnopqrstuvwxyz" in
-  let conv_filename = Charset.to_locale filename in
-  if filename <> conv_filename then Charset.conversion_enabled := false;
+  let conv_filename = Charset.Locale.to_locale filename in
+  if filename <> conv_filename then Charset.Locale.conversion_enabled := false;
 
-  Unix2.can_write_to_directory (Filename2.temp_directory ());
+  (try
+     ignore (Sys.getenv "MLDONKEY_TEMP")
+   with Not_found ->
+     Unix.putenv "MLDONKEY_TEMP" ((Filename.basename Sys.argv.(0)) ^ "_tmp")
+  );
+
+  Unix2.can_write_to_directory (Filename2.temp_dir_name ());
 
   if (String2.starts_with (Filename.basename Sys.argv.(0)) "mlnet") then begin
     if Sys.file_exists pid_filename then begin
@@ -297,7 +303,7 @@ let results_ini = create_options_file "results.ini"
 let files_ini = create_options_file "files.ini"
 let friends_ini = create_options_file "friends.ini"
 
-let messages_log = ref "messages.log"
+let messages_log = "messages.log"
 
 let servers_section = file_section servers_ini [] ""
 
@@ -446,7 +452,7 @@ let allowed_ips = define_option current_section ["allowed_ips"]
   or 192.168.0.0-192.168.0.255 for 192.168.0.*"
     ip_range_list_option [ Ip.RangeSingleIp Ip.localhost ]
 
-let allowed_ips_set = ref Ip_set.BL_Empty
+let allowed_ips_set = ref Ip_set.bl_empty
 
 let _ =
   option_hook allowed_ips (fun _ ->
@@ -478,7 +484,7 @@ let _ =
 let gui_port = define_option current_section ["gui_port"]
   ~desc: "The port to connect the GUI"
   ~restart: true
-  "port for Graphical Interfaces"
+  "port for Graphical Interfaces, 0 to deactivate GUI interface"
     port_option 4001
 
 let gift_port = define_option current_section ["gift_port"]
@@ -492,13 +498,13 @@ let http_port = define_option current_section ["http_port"]
   ~desc: "The port to connect via HTTP"
   ~public: true
   ~restart: true
-  "The port used to connect to your client with a WEB browser"
+  "The port used to connect to your client with a web browser, 0 to deactivate web interface"
     port_option 4080
 
 let telnet_port = define_option current_section ["telnet_port"]
   ~desc: "The port to connect via telnet"
   ~restart: true
-  "port for user interaction"
+  "port for user interaction, 0 to deactivate telnet interface"
     port_option 4000
 
 let http_bind_addr = define_expert_option current_section ["http_bind_addr"]
@@ -533,6 +539,15 @@ let alias_commands = define_option current_section ["alias_commands"]
       "exit", "q";
     ]
 
+let upnp_port_forwarding = define_option current_section ["upnp_port_forwarding"]
+  ~restart: true
+  "upnp port forwarding"
+    bool_option false
+
+let clear_upnp_port_at_exit = define_option current_section ["clear_upnp_port_at_exit"]
+  "clear all upnp port forwarding before mldonkey exit"
+    bool_option true
+
 let verbosity = define_expert_option current_section ["verbosity"]
   "A space-separated list of keywords. Each keyword triggers
   printing information on the corresponding messages:
@@ -563,7 +578,8 @@ let verbosity = define_expert_option current_section ["verbosity"]
   act : debug activity
   bw : debug bandwidth
   geo : debug GeoIP
-  unexp : debug unexpected messages"
+  unexp : debug unexpected messages
+  dht : debug DHT"
     string_option ""
 
 
@@ -603,17 +619,22 @@ let max_hard_download_rate_2 = define_option current_section ["max_hard_download
 let max_opened_connections = define_option current_section ["max_opened_connections"]
   "Maximal number of opened connections"
     int_option 200
+    
+let max_opened_connections_2 = define_option current_section ["max_opened_connections_2"]
+  "Second maximal number of opened connections for easy toggling (use bw_toggle)"
+    int_option 100
 
 let max_indirect_connections = define_option current_section ["max_indirect_connections"]
   "Amount of indirect connections in percent (min 30, max 70) of max_opened_connections"
     int_option 30
 
 let max_upload_slots = define_option current_section ["max_upload_slots"]
-  "How many slots can be used for upload"
+  "How many slots can be used for upload, minimum 3"
     int_option 5
 
 let max_release_slots = define_option current_section ["max_release_slots"]
-  "How many percent of upload slots can be used for downloading files tagged as release"
+  "How many percent of upload slots can be used for downloading files
+  tagged as release, maximum 75%"
     percent_option 20
 
 let friends_upload_slot = define_option current_section ["friends_upload_slot"]
@@ -1012,6 +1033,7 @@ let web_infos = define_option current_section ["web_infos"]
   'ocl' for file in the ocl format containing overnet peers, or
   'contact.dat' for an contact.dat file containing overnet peers,
   'nodes.gzip' for a fasttrack nodes.gzip,
+  'hublist' for DirectConnect hubs list,
   and period is the period between updates (in hours),
   a period of zero means the file is only loaded once on startup,
   and url is the url of the file to download.
@@ -1019,11 +1041,11 @@ let web_infos = define_option current_section ["web_infos"]
   EXAMPLE:
     web_infos = [
     (\"server.met\", 0, \"http://www.gruk.org/server.met.gz\");
-    (\"hublist\", 0, \"http://dchublist.com/hublist.config.bz2\");
+    (\"hublist\", 0, \"http://dchublist.com/hublist.xml.bz2\");
     (\"guarding.p2p\", 96, \"http://www.bluetack.co.uk/config/level1.gz\");
     (\"ocl\", 24, \"http://members.lycos.co.uk/appbyhp2/FlockHelpApp/contact-files/contact.ocl\");
     (\"contact.dat\", 168, \"http://download.overnet.org/contact.dat\");
-    (\"geoip.dat\", 168, \"http://www.maxmind.com/download/geoip/database/GeoIP.dat.gz\");
+    (\"geoip.dat\", 168, \"http://www.maxmind.com/download/geoip/database/GeoLiteCountry/GeoIP.dat.gz\");
     ]
   "
     (list_option (tuple3_option (string_option, int_option, string_option)))
@@ -1032,10 +1054,8 @@ let web_infos = define_option current_section ["web_infos"]
         "http://www.bluetack.co.uk/config/level1.gz");
       ("server.met", 0,
         "http://www.gruk.org/server.met.gz");
-      ("contact.dat", 168,
-        "http://download.overnet.org/contact.dat");
       ("geoip.dat", 0,
-        "http://www.maxmind.com/download/geoip/database/GeoIP.dat.gz");
+        "http://www.maxmind.com/download/geoip/database/GeoLiteCountry/GeoIP.dat.gz");
       ("nodes.gzip", 0,
         "http://update.kceasy.com/update/fasttrack/nodes.gzip");
       ("hublist", 0,
@@ -1118,13 +1138,19 @@ let http_proxy_port = define_option current_section ["http_proxy_port"]
   "Port of HTTP proxy"
     port_option 8080
 
+let http_proxy_login = define_option current_section ["http_proxy_login"]
+  "HTTP proxy login (leave empty if proxy doesn't require authentication)"
+    string_option ""
+
+let http_proxy_password = define_option current_section ["http_proxy_password"]
+  "HTTP proxy password"
+    string_option ""
+
 let http_proxy_tcp = define_option current_section ["http_proxy_tcp"]
   "Direct TCP connections to HTTP proxy (the proxy should support CONNECT)"
     bool_option false
 
 
-
-
 (*************************************************************************)
 (*                                                                       *)
 (*                         Mail section                                  *)
@@ -1134,27 +1160,35 @@ let http_proxy_tcp = define_option current_section ["http_proxy_tcp"]
 let current_section = mail_section
 
 let smtp_server = define_option current_section ["smtp_server"]
-  "The mail server you want to use (must be SMTP). Use hostname or IP address"
+  (_s"The mail server you want to use (must be SMTP). Use hostname or IP address")
     string_option "127.0.0.1"
 
 let smtp_port = define_option current_section ["smtp_port"]
-  "The port to use on the mail server (default 25)"
+  (_s"The port to use on the mail server (default 25)")
     port_option 25
 
+let smtp_login = define_option current_section ["smtp_login"]
+  (_s"Login to use for SMTP authentication (leave empty to disable). LOGIN, PLAIN and CRAM-MD5 methods are supported")
+  string_option ""
+
+let smtp_password = define_option current_section ["smtp_password"]
+  (_s"Password to use for SMTP authentication")
+  string_option ""
+
 let mail = define_option current_section ["mail"]
-  "Your e-mail if you want to receive mails when downloads are completed"
+  (_s"Email address to receive notifications when downloads are completed or disk is full (leave empty to disable, separate multiple addresses with space)")
     string_option ""
 
 let add_mail_brackets = define_option current_section ["add_mail_brackets"]
-  "Does your mail-server need <...> around addresses"
-    bool_option false
+  (_s"Set to false if your mail server cannot handle angle-brackets around addresses (RFC 5321)")
+    bool_option true
 
 let filename_in_subject = define_option current_section ["filename_in_subject"]
-  "Send filename in mail subject"
+  (_s"Send filename in mail subject")
     bool_option true
 
 let url_in_mail = define_option current_section ["url_in_mail"]
-  "Put a prefix for the filename here which shows up in the notification mail"
+  (_s"Put a prefix for the filename here which shows up in the notification mail")
     string_option ""
 
 
@@ -1178,6 +1212,10 @@ let pause_new_downloads = define_option current_section ["pause_new_downloads"]
   will be set to false on core start."
     bool_option false
 
+let release_new_downloads = define_option current_section ["release_new_downloads"]
+  "Set to true if you want to activate the release slot feature for all new downloads."
+    bool_option false
+
 (*  emulate_sparsefiles does not work, temporarily disabled
 let emulate_sparsefiles = define_expert_option current_section ["emulate_sparsefiles"]
   "Set to true if you want MLdonkey to emulate sparse files on your disk.
@@ -1201,11 +1239,11 @@ let max_recover_gap = define_option current_section ["max_recover_zeroes_gap"]
 
 let file_completed_cmd = define_option current_section ["file_completed_cmd"]
   "A command that is called when a file is committed, does not work on MinGW.
-  Arguments are (kept for compatability):
+  Arguments are (kept for compatibility):
     $1 - temp file name, without path
     $2 - file size
     $3 - filename of the committed file
-  Also these environment variables can be used (prefered way):
+  Also these environment variables can be used (preferred way):
     $TEMPNAME  - temp file name, including path
     $FILEID    - same as $1
     $FILESIZE  - same as $2
@@ -1215,12 +1253,27 @@ let file_completed_cmd = define_option current_section ["file_completed_cmd"]
     $INCOMING  - directory used for commit
     $NETWORK   - network used for downloading
     $ED2K_HASH - ed2k hash if MD4 is known
+    $FILE_OWNER - user who started the download
+    $FILE_GROUP - group the file belongs to
+    $USER_MAIL - mail address of file_owner
   "
     string_option ""
 
 let file_started_cmd = define_option current_section ["file_started_cmd"]
   "The command which is called when a download is started. Arguments
-  are '-file <num>'"
+  are '-file <num>'
+  Also these environment variables can be used (preferred way):
+    $TEMPNAME  - temp file name, including path
+    $FILEID    - same as $1
+    $FILESIZE  - same as $2
+    $FILENAME  - same as $3
+    $FILEHASH  - internal hash
+    $NETWORK   - network used for downloading
+    $ED2K_HASH - ed2k hash if MD4 is known
+    $FILE_OWNER - user who started the download
+    $FILE_GROUP - group the file belongs to
+    $USER_MAIL - mail address of file_owner
+  "
     string_option ""
 
 
@@ -1236,17 +1289,27 @@ let current_section = startup_section
 let run_as_user = define_option current_section ["run_as_user"]
   ~restart: true
   "The login of the user you want mldonkey to run as, after the ports
-  have been bound (can be use not to run with root priviledges when
+  have been bound (can be used not to run with root privileges when
   a port < 1024 is needed)"
     string_option ""
 
 let run_as_useruid = define_option current_section ["run_as_useruid"]
   ~restart: true
   "The UID of the user (0=disabled) you want mldonkey to run as, after the ports
-  have been bound (can be use not to run with root priviledges when
+  have been bound (can be used not to run with root privileges when
   a port < 1024 is needed)"
     int_option 0
 
+let run_as_group = define_option current_section ["run_as_group"]
+  ~restart: true
+  "The group of run_as_user user to be used"
+    string_option ""
+
+let run_as_groupgid = define_option current_section ["run_as_groupgid"]
+  ~restart: true
+  "The group of run_as_user user to be used"
+    int_option 0
+
 let ask_for_gui = define_option current_section ["ask_for_gui"]
   "Ask for GUI start"
     bool_option false
@@ -1295,7 +1358,7 @@ let create_dir_mode = define_option current_section ["create_dir_mode"]
     string_option "755"
 
 let create_file_sparse = define_option current_section ["create_file_sparse"]
-  "Create new files as sparse, only valid on MinGW for files on NTFS drives"
+  "Create new files as sparse (not supported on FAT volumes)"
     bool_option true
 
 let hdd_temp_minfree = define_option current_section ["hdd_temp_minfree"]
@@ -1459,6 +1522,10 @@ let max_name_len = define_expert_option current_section ["max_name_len"]
   "The size long names will be shortened to in the interface"
     int_option 50
 
+let max_result_name_len = define_expert_option current_section ["max_result_name_len"]
+  "The size filenames will be shortened to in search results"
+    int_option 50
+
 let max_filenames = define_expert_option current_section ["max_filenames"]
   "The maximum number of different filenames used by MLDonkey"
     int_option 50
@@ -1598,7 +1665,7 @@ let max_displayed_results = define_expert_option current_section ["max_displayed
 let options_version = define_expert_option current_section ["options_version"]
   ~internal: true
   "(internal option)"
-    int_option 20
+    int_option 22
 
 let max_comments_per_file = define_expert_option current_section ["max_comments_per_file"]
   "Maximum number of comments per file"
@@ -1698,14 +1765,13 @@ let _ =
   match name with
   | "mldc" -> enable_directconnect =:= true
   | "mlgnut" -> enable_gnutella =:= true
-  | "mldonkey" -> enable_donkey =:= true; enable_overnet =:= true
+  | "mldonkey" -> enable_donkey =:= true;
   | "mlslsk" -> enable_soulseek =:= true
   | "mlbt" -> enable_bittorrent =:= true
   | "mlnap" -> enable_opennap =:= true
   | _ ->
 (* default *)
       enable_donkey =:= true;
-      enable_overnet =:= true;
       enable_bittorrent =:= true
 
 let win_message =
@@ -1728,6 +1794,9 @@ let _ =
     end;
     calc_real_max_indirect_connections ()
   );
+  option_hook max_release_slots (fun _ ->
+    if !!max_release_slots > 75 then max_release_slots =:= 75
+  );
   option_hook min_reask_delay (fun _ ->
     if !!min_reask_delay < 600 then min_reask_delay =:= 600
   );
@@ -1768,9 +1837,6 @@ let _ =
           close_log ()
         end
   );
-  option_hook max_upload_slots (fun _ ->
-      if !!max_upload_slots < 3 then
-        max_upload_slots =:= 3);
   option_hook buffer_writes_threshold (fun _ ->
       Unix32.max_buffered := Int64.of_int (1024 * !!buffer_writes_threshold));
   option_hook log_size (fun _ ->
@@ -1841,6 +1907,7 @@ let verbose_activity = ref false
 let verbose_user_commands = ref false
 let verbose_geoip = ref false
 let verbose_unexpected_messages = ref false
+let verbose_dht = ref (ref false)
 
 let set_all v =
   verbose_msg_clients := v;
@@ -1870,7 +1937,8 @@ let set_all v =
   verbose_activity := v;
   verbose_user_commands := v;
   Geoip.verbose := v;
-  verbose_unexpected_messages := v
+  verbose_unexpected_messages := v;
+  !verbose_dht := v
 
 let _ =
   option_hook verbosity (fun _ ->
@@ -1907,6 +1975,7 @@ let _ =
           | "unexp" -> verbose_unexpected_messages := true
           | "com" -> verbose_user_commands := true
           | "geo" -> Geoip.verbose := true
+          | "dht" -> !verbose_dht := true
 
           | "all" ->
 
@@ -2007,14 +2076,20 @@ let http_proxy_tcp_update _ =
 
 let _ =
   let proxy_update _ =
-    http_proxy :=
-    (match !!http_proxy_server with
-        "" -> None
-      | _  -> Some (!!http_proxy_server, !!http_proxy_port));
+    let auth = match !!http_proxy_login with
+    | "" -> None
+    | _ -> Some (!!http_proxy_login, !!http_proxy_password)
+    in
+    http_proxy := 
+      (match !!http_proxy_server with
+      | "" -> None
+      | _  -> Some (!!http_proxy_server, !!http_proxy_port, auth));
     http_proxy_tcp_update ()
   in
   option_hook http_proxy_server proxy_update;
   option_hook http_proxy_port proxy_update;
+  option_hook http_proxy_login proxy_update;
+  option_hook http_proxy_password proxy_update;
   option_hook http_proxy_tcp http_proxy_tcp_update
 
 let _ =
@@ -2053,6 +2128,11 @@ let web_infos_add kind period url =
       state = None;
     }
 
+let web_infos_replace old_url new_url =
+  Hashtbl.iter (fun key w -> 
+    if w.url = old_url then w.url <- new_url
+  ) web_infos_table
+
 let _ =
 (* convert list option web_infos to a hashtable for better usage *)
   set_after_load_hook downloads_ini (fun _ ->
@@ -2179,4 +2259,15 @@ let rec update_options () =
       if !!share_scan_interval = 5 then share_scan_interval =:= 30;
       update 20
 
+  | 20 ->
+      web_infos_replace
+        "http://www.maxmind.com/download/geoip/database/GeoIP.dat.gz"
+        "http://www.maxmind.com/download/geoip/database/GeoLiteCountry/GeoIP.dat.gz";
+      update 21
+
+  | 21 ->
+      web_infos_remove "http://download.overnet.org/contact.dat";
+      enable_overnet =:= false;
+      update 22
+
   | _ -> ()
