@@ -103,7 +103,6 @@ let rec dollar_escape o with_frames s =
 
 let eval auth cmd o =
   let buf = o.conn_buf in
-  let cmd = Url.decode ~raw:false cmd in
   let cmd =
     if String2.check_prefix cmd "ed2k://" ||
        String2.check_prefix cmd "ftp://" ||
@@ -112,8 +111,9 @@ let eval auth cmd o =
     else cmd in
   let l = String2.tokens cmd in
   match l with
-    [] -> ()
-  | ["longhelp"] | ["??"] ->
+  | [] -> ()
+  | "longhelp"::subs | "??"::subs ->
+      let filter cmd = List.for_all (String2.contains cmd) subs in
       let module M = CommonMessages in
       if o.conn_output = HTML then begin
           Buffer.add_string buf "\\<div class=\\\"cs\\\"\\>";
@@ -125,8 +125,8 @@ let eval auth cmd o =
             ("", "srh", ""); ];
           Buffer.add_string buf "\\</tr\\>";
           html_mods_cntr_init ();
-          List.iter (fun (cmd, _, _, help) ->
-              let ncmd = ref cmd in
+          let show (cmd, _, _, help) =
+              let ncmd = ref cmd in (* why? *)
               let nhelp = ref help in
               Printf.bprintf buf "\\<tr class=\\\"dl-%d\\\"\\>" (html_mods_cntr ());
               html_mods_td buf [
@@ -136,9 +136,10 @@ let eval auth cmd o =
                 ("", "sr", "\\<a href=\\\"http://mldonkey.sourceforge.net/" ^ (String2.upp_initial !ncmd) ^
                   "\\\"\\>wiki\\</a\\>"); ];
               Printf.bprintf buf "\\</tr\\>\n";
-          )
-          (List.sort (fun (c1,_, _,_) (c2,_, _,_) -> compare c1 c2)
-            !CommonNetwork.network_commands);
+          in
+          List.iter show 
+            (List.sort (fun (c1,_, _,_) (c2,_, _,_) -> compare c1 c2)
+              (List.filter (fun (c,_,_,_) -> filter c) !CommonNetwork.network_commands));
           Printf.bprintf buf "\\</table\\>\\</div\\>";
           html_mods_table_header buf "helpTable" "results" [];
           Printf.bprintf buf "\\<tr class=\\\"dl-1\\\"\\>";
@@ -153,15 +154,15 @@ let eval auth cmd o =
           let list = Hashtbl2.to_list2 commands_by_kind in
           let list = List.sort (fun (s1,_) (s2,_) -> compare s1 s2) list in
           List.iter (fun (s,list) ->
-              Printf.bprintf buf "\n   $b%s$n:\n" s;
-              let list = List.sort (fun (s1,_) (s2,_) -> compare s1 s2) !list in
-              List.iter (fun (cmd, help) ->
-                  Printf.bprintf buf "$r%s$n %s\n" cmd help;
-              ) list
+              match List.sort (fun (s1,_) (s2,_) -> compare s1 s2) (List.filter (fun (s,_) -> filter s) !list) with
+              | [] -> ()
+              | list ->
+                Printf.bprintf buf "\n   $b%s$n:\n" s;
+                List.iter (fun (cmd, help) -> Printf.bprintf buf "$r%s$n %s\n" cmd help) list
           ) list;
         end
 
-    | ["help"] | ["?"] ->
+    | ["help"] | ["?"] | ["man"] ->
           let module M = CommonMessages in
            if o.conn_output = HTML then
              begin
@@ -251,7 +252,7 @@ let eval auth cmd o =
                html_mods_td buf [
                  ("", "sr", "Use '$r\\<a href=\\\"submit?q=longhelp\\\"\\>" ^
                    "longhelp\\</a\\>$n' or '$r\\<a href=\\\"submit?q=longhelp\\\"\\>" ^
-                   "??\\</a\\>$n' for all commands."); ];
+                   "??\\</a\\>$n' for all commands. Specify substring to filter."); ];
                Buffer.add_string buf "\\</tr\\>\\<tr class=\\\"dl-1\\\"\\>";
                html_mods_td buf [
                  ("", "sr", "Use '$rhelp command$n' or '$r? command$n' for help on a command."); ];
@@ -283,14 +284,14 @@ $bGeneral:$n
           $rq$n : quit this interface
 
 Use '$rlonghelp$n' or '$r??$n' for all commands.
+Use '$rlonghelp str$n' or '$r?? str$n' for all commands that contain specified substring.
 Use '$rhelp command$n' or '$r? command$n' for help on a command.
             ";
     | "?" :: args | "help" :: args | "man" :: args ->
           List.iter (fun arg ->
-              List.iter (fun (cmd, _, _, help) ->
-                  if cmd = arg then
-                    Printf.bprintf  buf "%s %s\n" cmd help)
-              !CommonNetwork.network_commands)
+              match List.filter (fun (cmd, _, _, _) -> cmd = arg) !CommonNetwork.network_commands with
+              | [] -> Printf.bprintf buf "Unknown command : %s\n" arg
+              | l -> List.iter (fun (_,_,_,help) -> Printf.bprintf  buf "%s %s\n" arg help) l)
           args
     | one :: two ->
         let cmd, args =
@@ -585,7 +586,7 @@ let telnet_handler t event =
             conn_info = Some (TELNET, (from_ip, from_port));
           } in
 	(match Ip_set.match_ip !allowed_ips_set from_ip with
-	| Some br -> 
+	| true -> 
         TcpBufferedSocket.prevent_close sock;
         TcpBufferedSocket.set_max_output_buffer sock !!interface_buffer;
         TcpBufferedSocket.set_reader sock (user_reader o telnet);
@@ -604,7 +605,7 @@ let telnet_handler t event =
 
         after_telnet_output o sock
 
-	| None ->
+	| false ->
         before_telnet_output o sock;
 	let reject_message =
 	  Printf.sprintf "Telnet connection from %s rejected (see allowed_ips setting)\n"
@@ -767,7 +768,7 @@ let read_theme_page page =
     s)
 
 let http_add_gen_header r =
-  add_reply_header r "Server" "MLdonkey";
+  add_reply_header r "Server" ("MLdonkey/"^Autoconf.current_version);
   add_reply_header r "Connection" "close"
 
 let add_gzip_headers r =
@@ -793,6 +794,7 @@ let http_add_text_header r ext =
 let http_add_bin_info_header r clen =
   add_reply_header r "Accept-Ranges" "bytes";
   add_reply_header r "Content-Length" (Printf.sprintf "%d" clen)
+  (* FIXME Content-Length is duplicated *)
 
 let http_add_bin_header r ext clen =
   http_file_type := ext_to_file_type ext;
@@ -816,16 +818,25 @@ let http_send_bin r buf filename =
     else
       try
         File.to_string filename
-      with _ ->
-        try
-          Hashtbl.find CommonPictures.files filename
-        with Not_found ->
-          try
-            if String.sub filename 0 4 = "flag" then
-              Hashtbl.find CommonPictures.files "flag_--.png"
-            else
-              raise Not_found
-          with _ -> raise Not_found
+      with _ -> raise Not_found
+  in
+  let ext = extension_to_file_ext (Filename2.last_extension2 filename) in
+  http_add_bin_header r ext (String.length file_to_send);
+  add_reply_header r "Cache-Control" "no-cache";
+  add_reply_header r "Pragma" "no-cache";
+  Buffer.add_string buf file_to_send
+
+let http_send_bin_pictures r buf filename =
+  let file_to_send =
+    try
+      Hashtbl.find CommonPictures.files filename
+    with Not_found ->
+      try
+        if String.sub filename 0 4 = "flag" then
+          Hashtbl.find CommonPictures.files "flag_--.png"
+        else
+          raise Not_found
+    with _ -> raise Not_found
   in
   let ext = extension_to_file_ext (Filename2.last_extension2 filename) in
   http_add_bin_header r ext (String.length file_to_send);
@@ -1391,6 +1402,11 @@ let http_handler o t r =
                   html_open_page buf t r true;
                   send_custom_query o.conn_user buf query args
 
+              | [ ("api", cmd) ] ->
+                  clear_page o.conn_buf;
+                  eval (ref true) cmd o;
+                  Buffer.add_string buf (Buffer.contents o.conn_buf)
+
               | [ "setoption", _ ; "option", name; "value", value ] ->
                   html_open_page buf t r true;
                   let gui_type, ip, port =
@@ -1501,7 +1517,7 @@ let http_handler o t r =
                 read_theme_page this_page else
               if !!html_mods then !!CommonMessages.download_html_js_mods0
               else !!CommonMessages.download_html_js_old)
-        | s ->  http_send_bin r buf (String.lowercase s)
+        | s ->  http_send_bin_pictures r buf (String.lowercase s)
       with
       | Not_found ->
 	  let _, error_text_long, header = Http_server.error_page "404" "" ""
