@@ -153,6 +153,8 @@ let current_files = ref ([] : BTTypes.file list)
 
 let listen_sock = ref (None : TcpServerSocket.t option)
 
+let bt_dht = ref (None : BT_DHT.M.t option)
+
 let files_by_uid = Hashtbl.create 13
 
 let max_range_len = Int64.of_int (1 lsl 14)
@@ -194,7 +196,7 @@ let check_if_interesting file c =
       (match c.client_chunk with
       | None -> true
       | Some (chunk, blocks) ->
-	  List.for_all (fun b ->
+          List.for_all (fun b ->
             let chunk_num = CommonSwarming.block_chunk_num swarmer b.up_block in
             let bitmap = CommonSwarming.chunks_verified_bitmap swarmer in
             VB.get bitmap chunk_num <> VB.State_verified) blocks)
@@ -230,17 +232,28 @@ let create_temp_file file_temp file_files file_state =
         file_temp);
   file_fd
 
-let can_handle_tracker t =
-  String2.check_prefix (String.lowercase t.tracker_url) "http://"
+let make_tracker_url url =
+  if String2.check_prefix (String.lowercase url) "http://" then 
+    `Http url (* do not change the case of the url *)
+  else
+    try Scanf.sscanf (String.lowercase url) "udp://%s@:%d" (fun host port -> `Udp (host,port))
+    with _ -> `Other url
 
-let rec set_trackers file file_trackers =
-  match file_trackers with
-    | [] -> ()
-    | url :: q ->
-	if not (List.exists (fun tracker -> 
-			       tracker.tracker_url = url
-			    ) file.file_trackers) then 
-	  let t = {
+(** invariant: [make_tracker_url (show_tracker_url url) = url] *)
+let show_tracker_url : tracker_url -> string = function
+  | `Http url | `Other url -> url
+  | `Udp (host,port) -> Printf.sprintf "udp://%s:%d" host port
+
+let can_handle_tracker = function
+  | `Http _
+  | `Udp _ -> true
+  | `Other _ -> false
+
+let set_trackers file file_trackers =
+  List.iter (fun url ->
+        let url = make_tracker_url url in
+        if not (List.exists (fun tracker -> tracker.tracker_url = url) file.file_trackers) then
+          let t = {
             tracker_url = url;
             tracker_interval = 600;
             tracker_min_interval = 600;
@@ -253,14 +266,13 @@ let rec set_trackers file file_trackers =
             tracker_torrent_last_dl_req = 0;
             tracker_id = "";
             tracker_key = "";
-	    tracker_status = Enabled
+            tracker_status = if can_handle_tracker url then Enabled 
+                             else Disabled_mld (intern "Tracker type not supported")
           } in
-	  if not (can_handle_tracker t) then
-	    t.tracker_status <- Disabled_mld (intern "Tracker type not supported");
-	  file.file_trackers <-  t :: file.file_trackers;
-	set_trackers file q
+          file.file_trackers <-  t :: file.file_trackers)
+  file_trackers
 
-let new_file file_id t torrent_diskname file_temp file_state user =
+let new_file file_id t torrent_diskname file_temp file_state user group =
   try
     Hashtbl.find files_by_uid file_id
   with Not_found ->
@@ -287,10 +299,14 @@ let new_file file_id t torrent_diskname file_temp file_state user =
           file_torrent_diskname = torrent_diskname;
           file_completed_hook = (fun _ -> ());
           file_shared = None;
+          file_session_uploaded = Int64.zero;
+          file_session_downloaded = Int64.zero;
+          file_last_dht_announce = 0;
+          file_private = t.torrent_private;
         } and file_impl =  {
           dummy_file_impl with
           impl_file_owner = user;
-          impl_file_group = user.user_default_group;
+          impl_file_group = group;
           impl_file_fd = Some file_fd;
           impl_file_size = t.torrent_length;
           impl_file_downloaded = Int64.zero;
@@ -325,7 +341,7 @@ let new_file file_id t torrent_diskname file_temp file_state user =
                           if not (Bitv.get bitmap num) then
                             send_client c (Have (Int64.of_int num));
                           check_if_interesting file c
-                    end       
+                    end
               ) file.file_clients
 
           );
@@ -410,9 +426,9 @@ let dot_string_of_string s =
   String.iter (fun s ->
     match s with
     | '0' .. '9' ->
-	if !found_non_int then Buffer.add_char buf '.';
-	found_non_int := false;
-	Buffer.add_char buf s
+        if !found_non_int then Buffer.add_char buf '.';
+        found_non_int := false;
+        Buffer.add_char buf s
     | _ -> found_non_int := true
   ) s;
   Buffer.contents buf
@@ -465,6 +481,7 @@ let decode_az_style s =
       | "ZT" -> Brand_ziptorrent
       | "bk" -> Brand_bitkitten
       | "MP" -> Brand_moopolice
+      | "UM" -> Brand_utorrent_mac
       | "UT" -> Brand_utorrent
       | "KT" -> Brand_ktorrent
       | "LP" -> Brand_lphant
@@ -482,9 +499,15 @@ let decode_az_style s =
       | "AX" -> Brand_bitpump
       | "DE" -> Brand_deluge
       | "TT" -> Brand_tuotu
+      | "SD" (* Thunder (aka XùnLéi) *)
       | "XL" -> Brand_xunlei
       | "FT" -> Brand_foxtorrent
       | "BF" -> Brand_bitflu
+      | "OS" -> Brand_oneswarm
+      | "LW" -> Brand_limewire
+      | "HL" -> Brand_halite
+      | "MR" -> Brand_miro
+      | "PD" -> Brand_pando
       | _ -> Brand_unknown
     in
     if brand = Brand_unknown then None else
@@ -493,12 +516,15 @@ let decode_az_style s =
           (* 4.56 *)
           | Brand_bitpump
           | Brand_bitcomet -> (String.sub s 4 1) ^ "." ^ (String.sub s 5 2)
-          (* 3.45 *)
+          (* 3.4.5 *)
           | Brand_tuotu
+          | Brand_utorrent_mac
+          | Brand_oneswarm
           | Brand_utorrent -> (String.sub s 3 1) ^ "." ^ (String.sub s 4 1) ^ "." ^ (String.sub s 5 1)
+          (* 3.45 *)
+          | Brand_transmission -> (String.sub s 3 1) ^ "." ^ (String.sub s 4 2)
           (* 34.56 *)
-          | Brand_ctorrent
-          | Brand_transmission -> (strip_leading_zeroes (String.sub s 3 2)) ^ "." ^ (strip_leading_zeroes(String.sub s 5 2))
+          | Brand_ctorrent -> (strip_leading_zeroes (String.sub s 3 2)) ^ "." ^ (strip_leading_zeroes(String.sub s 5 2))
           (* 3.4.5->[R=RC.6|D=Dev|''] *)
           | Brand_ktorrent -> 
               let x = match s.[5] with 
@@ -594,6 +620,7 @@ let decode_simple_style s =
       (0,  "btpd",              Brand_btpd,            (dot_string(String.sub s 5 3)));
       (0,  "XBT",               Brand_xbt,             (dot_string(String.sub s 3 3)));
       (0,  "-FG",               Brand_flashget,        (dot_string(String.sub s 4 3)));
+      (0,  "-SP",               Brand_bitspirit,       (dot_string(String.sub s 3 3)));
     ]
   in
   let len = List.length !simple_list in
@@ -757,16 +784,17 @@ let decoder_list = [
 
 let parse_software s =
   let default = (Brand_unknown, "") in
-  let rec iter l =
-    match l with
-      [] -> lprintf_nl "Unknown BT client software version, report the next line to http://mldonkey.sourceforge.net/UnknownBtClients%s\nBTUC:\"%s\"" Autoconf.current_version (String.escaped s);
-            default
-      | d :: t -> match (d s) with 
-                  | None    -> iter t
-                  | Some bv -> let (brand, version) = bv in
-                               if !verbose_msg_clienttags then
-                                 lprintf_nl "BTKC:\"%s\"; ID: \"%s\"; version:\"%s\"" (String.escaped s) (brand_to_string brand) version;
-                               bv
+  let rec iter = function
+  | [] ->
+    if !verbose_msg_clienttags then lprintf_nl "BTUC: %S" s;
+    default
+  | d :: t ->
+    match (d s) with
+    | None -> iter t
+    | Some (brand, version as bv) ->
+      if !verbose_msg_clienttags then
+        lprintf_nl "BTKC: %S; ID: %S; version: %S" s (brand_to_string brand) version;
+      bv
   in
   if Sha1.direct_of_string s = Sha1.null then
     default
@@ -865,7 +893,7 @@ let remove_client c =
 let remove_tracker url file =
   if !verbose_msg_servers then
     List.iter (fun tracker ->
-      lprintf_nl "Old tracker list :%s" tracker.tracker_url
+      lprintf_nl "Old tracker list: %s" (show_tracker_url tracker.tracker_url)
     ) file.file_trackers;
   List.iter (fun bad_tracker ->
     if bad_tracker.tracker_url = url then
@@ -873,7 +901,7 @@ let remove_tracker url file =
   ) file.file_trackers;
   if !verbose_msg_servers then
     List.iter (fun tracker ->
-      lprintf_nl "New tracker list :%s" tracker.tracker_url
+      lprintf_nl "New tracker list: %s" (show_tracker_url tracker.tracker_url)
     ) file.file_trackers
 
 let tracker_is_enabled t =
@@ -897,7 +925,7 @@ Define a function to be called when the "mem_stats" command
 
 **************************************************************)
 
-let _ =
+let () =
   Heap.add_memstat "BittorrentGlobals" (fun level buf ->
      Printf.bprintf buf "Number of old files: %d\n" (List.length !!old_files);
      let downloads = ref 0 in
@@ -912,3 +940,28 @@ let _ =
      Printf.bprintf buf "files_by_uid: %d\n" (Hashtbl.length files_by_uid);
      Printf.bprintf buf "ft_by_num: %d\n" (Hashtbl.length ft_by_num);
   )
+
+open BT_DHT
+
+let () =
+  Heap.add_memstat "BittorrentDHT" (fun _level buf ->
+    match !bt_dht with
+    | None -> ()
+    | Some dht ->
+    let (buckets,nodes,keys,peers) = stat dht in
+    Printf.bprintf buf "Routing : %d nodes in %d buckets\n" nodes buckets;
+    Printf.bprintf buf "Storage : %d keys with %d peers\n" keys peers;
+    List.iter (fun s -> Printf.bprintf buf "%s\n" s) (rpc_stats dht);
+    let queries = ["PING",`Ping;"FIND_NODE",`FindNode;"GET_PEERS",`GetPeers;"ANNOUNCE",`Announce] in
+    Printf.bprintf buf "Outgoing queries : ok/error/timeout\n";
+    List.iter begin fun (name,qt) ->
+      let get k = try Hashtbl.find dht.M.stats (qt,`Out k) with Not_found -> 0 in
+      Printf.bprintf buf "%s: %d/%d/%d\n" name (get `Answer) (get `Error) (get `Timeout);
+      end queries;
+    Printf.bprintf buf "Incoming queries\n";
+    List.iter begin fun (name,qt) ->
+      let get () = try Hashtbl.find dht.M.stats (qt,`In) with Not_found -> 0 in
+      Printf.bprintf buf "%s: %d\n" name (get ())
+      end queries
+  )
+
