@@ -40,7 +40,7 @@ type torrent = {
     mutable torrent_creation_date : int64;
     mutable torrent_modified_by : string;
     mutable torrent_encoding : string;
-    mutable torrent_private : int64;
+    mutable torrent_private : bool;
 (*
     mutable torrent_nodes : string;
 *)
@@ -92,6 +92,7 @@ type brand =
 | Brand_xbt
 | Brand_ziptorrent
 | Brand_moopolice
+| Brand_utorrent_mac
 | Brand_utorrent
 | Brand_opera
 | Brand_torrenttopia
@@ -120,6 +121,14 @@ type brand =
 | Brand_bitpump
 | Brand_deluge
 | Brand_btuga
+| Brand_tuotu
+| Brand_xunlei
+| Brand_foxtorrent
+| Brand_bitflu
+| Brand_oneswarm
+| Brand_halite
+| Brand_miro
+| Brand_pando
 
 let brand_list = [ 
    ( Brand_unknown          , "unknown"                , "unk" ) ;
@@ -167,6 +176,7 @@ let brand_list = [
    ( Brand_xbt              , "XBT"                    , "xbt" ) ;
    ( Brand_ziptorrent       , "ZipTorrent"             , "zit" ) ;
    ( Brand_moopolice        , "MooPolice"              , "moo" ) ;
+   ( Brand_utorrent_mac     , "uTorrent for Mac"       , "uTm" ) ;
    ( Brand_utorrent         , "uTorrent"               , "uTo" ) ;
    ( Brand_opera            , "Opera"                  , "opr" ) ;
    ( Brand_torrenttopia     , "TorrentTopia"           , "tt" ) ;
@@ -195,6 +205,14 @@ let brand_list = [
    ( Brand_bitpump          , "BitPump"                , "bpu" ) ;
    ( Brand_deluge           , "Deluge"                 , "del" ) ;
    ( Brand_btuga            , "BTuga Revolution"       , "btr" ) ;
+   ( Brand_tuotu            , "Tuotu"                  , "tuo" ) ;
+   ( Brand_xunlei           , "XunLei"                 , "xun" ) ;
+   ( Brand_foxtorrent       , "FoxTorrent"             , "fox" ) ;
+   ( Brand_bitflu           , "BitFlu"                 , "flu" ) ;
+   ( Brand_oneswarm         , "OneSwarm"               , "osw" ) ;
+   ( Brand_halite           , "Halite"                 , "hal" ) ;
+   ( Brand_miro             , "Miro"                   , "mir" ) ;
+   ( Brand_pando            , "Pando"                  , "pan" ) ;
   ] 
 
 let brand_count = List.length brand_list
@@ -214,12 +232,18 @@ type tracker_status =
 | Disabled_mld of string
 | Disabled_failure of (int * string)
 
+type tracker_url =
+[ `Http of string (* url *)
+| `Udp of string * int (* host and port *)
+| `Other of string ]
+
 type client = {
     client_client : client CommonClient.client_impl;
     mutable client_file : file;
     mutable client_connection_control : connection_control;
     mutable client_sock : tcp_connection;
     mutable client_host : Ip.t * int;
+    mutable client_country_code : int option;
     mutable client_chunks : (int64 * int64) list;
     mutable client_uploader : CommonSwarming.uploader option;
     mutable client_ranges_sent : (int64 * int64 * CommonSwarming.range) list;
@@ -268,7 +292,7 @@ type client = {
   }
 
 and tracker_info = {
-    tracker_url : string;
+    tracker_url : tracker_url;
     mutable tracker_interval : int;
     mutable tracker_min_interval : int;
     mutable tracker_last_conn : int;
@@ -306,6 +330,12 @@ and file = {
     mutable file_tracker_connected : bool;
     mutable file_completed_hook : (file -> unit);
     mutable file_shared : file CommonShared.shared_impl option;
+    (** session uploaded and downloaded bytes, for statistics reporting *)
+    mutable file_session_uploaded : int64;
+    mutable file_session_downloaded : int64;
+    (** DHT specific *)
+    mutable file_last_dht_announce : int;
+    file_private : bool;
   }
 
 and ft = {
