@@ -276,7 +276,7 @@ let new_result file_name file_size tags hashes _ =
 
 let min_range_size = megabyte
 
-let new_file file_temporary file_name file_size file_hash user =
+let new_file file_temporary file_name file_size file_hash user group =
   let file_temp = Filename.concat !!temp_directory file_temporary in
 (*      (Printf.sprintf "FT-%s" (Md4.to_string file_id)) in *)
   let t = Unix32.create_rw file_temp in
@@ -303,7 +303,7 @@ let new_file file_temporary file_name file_size file_hash user =
       impl_file_size = file_size;
       impl_file_downloaded = Int64.zero;
       impl_file_owner = user;
-      impl_file_group = user.user_default_group;
+      impl_file_group = group;
       impl_file_val = file;
       impl_file_ops = file_ops;
       impl_file_age = last_time ();
@@ -346,7 +346,7 @@ let new_file file_temporary file_name file_size file_hash user =
 
 exception FileFound of file
 
-let new_file file_id file_name file_size file_uids user =
+let new_file file_id file_name file_size file_uids user group =
   let file = ref None in
   List.iter (fun uid ->
       match Uid.to_uid uid with
@@ -354,7 +354,7 @@ let new_file file_id file_name file_size file_uids user =
           file := Some (try
               Hashtbl.find files_by_uid file_hash
             with _ ->
-                let file = new_file file_id file_name file_size file_hash user in
+                let file = new_file file_id file_name file_size file_hash user group in
                 Hashtbl.add files_by_uid file_hash file;
                 file)
       | _ -> ()
