@@ -14,7 +14,7 @@
 
     You should have received a copy of the GNU General Public License
     along with mldonkey; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA
 */
 
 #include "../../utils/lib/os_stubs.h"
@@ -936,16 +936,16 @@ value ml_has_pthread(value unit)
 #define NTFS_SUPER_MAGIC      0x5346544E
 
 struct statfs {
-   long    f_type;     /* type of filesystem (see below) */
-   long    f_bsize;    /* optimal transfer block size */
-   long    f_blocks;   /* total data blocks in file system */
-   long    f_bfree;    /* free blocks in fs */
-   long    f_bavail;   /* free blocks avail to non-superuser */
-   long    f_files;    /* total file nodes in file system */
-   long    f_ffree;    /* free file nodes in fs */
-   long    f_fsid;     /* file system id */
-   long    f_namelen;  /* maximum length of filenames */
-   long    f_spare[6]; /* spare for later */
+   unsigned __int64    f_type;     /* type of filesystem (see below) */
+   unsigned __int64    f_bsize;    /* optimal transfer block size */
+   unsigned __int64    f_blocks;   /* total data blocks in file system */
+   unsigned __int64    f_bfree;    /* free blocks in fs */
+   unsigned __int64    f_bavail;   /* free blocks avail to non-superuser */
+   unsigned __int64    f_files;    /* total file nodes in file system */
+   unsigned __int64    f_ffree;    /* free file nodes in fs */
+   unsigned __int64    f_fsid;     /* file system id */
+   unsigned __int64    f_namelen;  /* maximum length of filenames */
+   unsigned __int64    f_spare[6]; /* spare for later */
 };
 
 static int statfs (const unsigned char *path, struct statfs *buf)
@@ -1072,9 +1072,13 @@ statfs_statfs (value pathv)
 #  define HAVE_STATS 1
 #endif  /* HAVE_SYS_VFS_H */
 
+#ifdef HAVE_SYS_STATVFS_H
+#  include <sys/statvfs.h>
+#endif
+
 #ifdef HAVE_STATS
 static value
-#if ((defined (sun) || defined (__sun__))) || (defined(__NetBSD__) && (__NetBSD_Version__ > 299000000)) || defined (__hpux__)
+#if ((defined (sun) || defined (__sun__))) || (defined(__NetBSD__) && (__NetBSD_Version__ > 299000000)) || defined (__hpux__) || defined(__alpha__)
 copy_statfs (struct statvfs *buf)
 #else
 copy_statfs (struct statfs *buf)
@@ -1083,7 +1087,7 @@ copy_statfs (struct statfs *buf)
   CAMLparam0 ();
   CAMLlocal2 (bufv, v);
   bufv = caml_alloc (11, 0);
-#if ((defined (sun) || defined (__sun__))) || (defined(__FreeBSD__) && __FreeBSD_version >= 503001) || defined(__OpenBSD__) || defined(__NetBSD__)
+#if ((defined (sun) || defined (__sun__))) || (defined(__FreeBSD__) && __FreeBSD_version >= 503001) || defined(__OpenBSD__) || defined(__NetBSD__) || defined(__alpha__)
   v = copy_int64 (-1); caml_modify (&Field (bufv, 0), v);
 #else
   v = copy_int64 (buf->f_type); caml_modify (&Field (bufv, 0), v);
@@ -1094,13 +1098,17 @@ copy_statfs (struct statfs *buf)
   v = copy_int64 (buf->f_bavail); caml_modify (&Field (bufv, 4), v);
   v = copy_int64 (buf->f_files); caml_modify (&Field (bufv, 5), v);
   v = copy_int64 (buf->f_ffree); caml_modify (&Field (bufv, 6), v);
-#if ((defined (sun) || defined (__sun__))) || defined (__hpux__)
+#if ((defined (sun) || defined (__sun__))) || defined (__hpux__) || defined(__alpha__)
   v = copy_int64 (-1); caml_modify (&Field (bufv, 7), v);
   v = copy_int64 (buf->f_namemax); caml_modify (&Field (bufv, 8), v);
+# if ! defined(__alpha__)
   v = copy_string (buf->f_basetype); caml_modify (&Field (bufv, 9), v);
+# else
+  v = copy_string ("-1"); caml_modify (&Field (bufv, 9), v);
+# endif
   v = copy_int64 (buf->f_frsize); caml_modify (&Field (bufv, 10), v);
 #else
-#if defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__NetBSD__) || defined(__APPLE__) || defined(__DragonFly__)
+#if defined(__FreeBSD__) || defined(__OpenBSD__) || defined(__NetBSD__) || defined(__APPLE__) || defined(__DragonFly__) || defined(__FreeBSD_kernel__)
 #  if defined(__OpenBSD__) || defined(__NetBSD__) || (defined(__FreeBSD__) && __FreeBSD_version < 502000) || defined(__DragonFly__) || defined(__APPLE__)
 #    include <sys/syslimits.h>
      v = copy_int64 (NAME_MAX); caml_modify (&Field (bufv, 8), v);
@@ -1126,7 +1134,7 @@ statfs_statfs (value pathv)
   CAMLparam1 (pathv);
   CAMLlocal1 (bufv);
   const char *path = String_val (pathv);
-#if ((defined (sun) || defined (__sun__))) || (defined(__NetBSD__) && (__NetBSD_Version__ > 299000000)) || defined (__hpux__)
+#if ((defined (sun) || defined (__sun__))) || (defined(__NetBSD__) && (__NetBSD_Version__ > 299000000)) || defined (__hpux__) || defined(__alpha__)
   struct statvfs buf;
   if (statvfs (path, &buf) == -1)
 #else
@@ -1475,3 +1483,25 @@ value ml_os_supported(value unit)
 	  return Val_false;
 }
 
+/*
+ * fsync
+ */
+CAMLprim value ml_fsync(value v_fd)
+{
+    CAMLparam1(v_fd);
+    int r = 0;
+    OS_FD fd;
+
+    if (OS_IS_FD(v_fd)) 
+      fd = Fd_val(v_fd); 
+    else 
+      caml_invalid_argument("fsync");
+
+    caml_enter_blocking_section();
+    r = os_fsync(fd);
+    caml_leave_blocking_section();
+    if (0 != r)
+      uerror("fsync",Nothing);
+    CAMLreturn(Val_unit);
+}
+
