@@ -27,6 +27,7 @@ open BasicSocket
 open Unix
 open Url
 open TcpBufferedSocket
+open Int64ops
 
 
 type http_request =
@@ -37,24 +38,33 @@ type http_request =
 | DELETE
 | TRACE
 
+type error = [ `HTTP of int | `RST of BasicSocket.close_reason | `DNS ]
+
+let show_error = function
+| `HTTP code -> Printf.sprintf "HTTP error code %d" code
+| `RST reason -> Printf.sprintf "Connection closed : %s" (BasicSocket.string_of_reason reason)
+| `DNS -> Printf.sprintf "DNS resolution failed"
+
 let verbose = ref false
 
 type request = {
     req_headers : ( string * string ) list;
     req_user_agent : string;
     req_accept : string;
-    req_proxy : (string * int) option;
+    req_proxy : (string * int * (string * string) option) option; (* (host,port,(login,password)) *)
     mutable req_url : url;
+    mutable req_gzip : bool;
     mutable req_save_to_file_time : float;
     req_request : http_request;
     req_referer : Url.url option;
     req_retry : int;
     req_max_retry : int;
     req_save : bool;
+    req_max_total_time : float;
   }
 
 type content_handler = 
-  int -> (string * string) list -> TcpBufferedSocket.t -> int -> unit
+  int64 -> (string * string) list -> TcpBufferedSocket.t -> int -> unit
 
 let log_prefix = "[HTTPcl]"
 
@@ -66,6 +76,7 @@ let basic_request = {
     req_referer = None;
     req_save_to_file_time = 0.;
     req_request = GET;
+    req_gzip = false;
     req_proxy = None;
     req_headers = [];
     req_user_agent = "Wget 1.4";
@@ -73,6 +84,7 @@ let basic_request = {
     req_retry = 0;
     req_max_retry = 0;
     req_save = false;
+    req_max_total_time = infinite_timeout;
   }
       
 let make_full_request r =
@@ -100,12 +112,19 @@ let make_full_request r =
   List.iter (fun (a,b) ->
       Printf.bprintf res "%s: %s\r\n" a b
   ) r.req_headers;
+  Printf.bprintf res "Accept-Encoding: gzip\r\n";
   Printf.bprintf res "User-Agent: %s\r\n" r.req_user_agent;
   Printf.bprintf res "Accept: %s\r\n" r.req_accept;
   Printf.bprintf res "Connection: close\r\n";
- (match r.req_referer with None -> ()
-    | Some url -> 
-        Printf.bprintf res "Referer: %s\r\n" (Url.to_string_no_args url));
+  begin match r.req_referer with 
+  | None -> ()
+  | Some url -> Printf.bprintf res "Referer: %s\r\n" (Url.to_string_no_args url)
+  end;
+  begin match r.req_proxy with
+  | Some (_,_,Some (login,password)) ->
+      Printf.bprintf res "Proxy-Authorization: Basic %s\n" (Base64.encode (login ^ ":" ^ password))
+  | _ -> ()
+  end;
   if url.user <> "" then begin
     let userpass = Printf.sprintf "%s:%s" url.user url.passwd in
     Printf.bprintf res "Authorization: Basic %s\r\n" (Base64.encode userpass)
@@ -206,9 +225,14 @@ let http_reply_handler nr headers_handler sock nread =
   read_header (parse_header headers_handler) sock nread
   
 
-let def_ferr = (fun c -> ())
+let def_ferr = (fun _ -> ())
 
 let rec get_page r content_handler f ferr =
+  let ok = ref false in
+  let ferr =
+    let err_done = ref false in (* call not more than once *)
+    fun c -> if not !err_done then begin err_done := true; ferr c; end 
+  in
   let rec get_url level r =
   try
     let url = r.req_url in
@@ -217,35 +241,21 @@ let rec get_page r content_handler f ferr =
     let server, port =
       match r.req_proxy with
       | None -> url.server, url.port
-      | Some (s, p) -> s, p
+      | Some (s, p, _) -> s, p
     in
 (*    lprintf "async_ip ...\n"; *)
     Ip.async_ip server (fun ip ->
 (*         lprintf "IP done %s:%d\n" (Ip.to_string ip) port;*)
         let token = create_token unlimited_connection_manager in
         let sock = TcpBufferedSocket.connect token "http client connecting"
-        (try Ip.to_inet_addr ip with e -> raise Not_found) port
-        (fun sock e -> 
-              () 
-(*              if !verbose then
-                  lprintf "Event %s\n"
-                    (match e with
-                      CONNECTED -> "CONNECTED"
-                    | WRITE_DONE -> "WRITE_DONE"
-                    | CAN_REFILL -> "CAN_REFILL"
-                    | BUFFER_OVERFLOW -> "BUFFER_OVERFLOW"
-                    | READ_DONE n -> Printf.sprintf "READ_DONE %d" n
-                    | BASIC_EVENT e ->
-                        match e with
-                          (CLOSED s) -> Printf.sprintf "CLOSED %s" (string_of_reason s)
-                        | RTIMEOUT -> "RTIMEOUT"
-                        | LTIMEOUT -> "LTIMEOUT"
-                        | WTIMEOUT -> "WTIMEOUT"
-                        | CAN_READ -> "CAN_READ"
-                        | CAN_WRITE -> "CAN_WRITE"
-                    )
- *)
-          )
+          (try Ip.to_inet_addr ip with e -> raise Not_found) port
+          (fun sock e -> 
+(*             if !verbose then lprintf_nl "Event %s" (string_of_event e); *)
+            match e with (* FIXME content-length check *)
+            | BASIC_EVENT (CLOSED (Closed_by_user | Closed_by_peer)) when !ok -> f ()
+            | BASIC_EVENT (CLOSED reason) -> ferr (`RST reason)
+            | BASIC_EVENT LTIMEOUT -> close sock Closed_for_lifetime
+            | _ -> ())
         in
 
         let nread = ref false in
@@ -255,8 +265,9 @@ let rec get_page r content_handler f ferr =
         TcpBufferedSocket.set_reader sock (http_reply_handler nread
             (default_headers_handler url level));
         set_rtimeout sock 5.;
+        set_lifetime sock r.req_max_total_time;
     )
-    ferr;
+    (fun () -> ferr `DNS);
   with e -> 
     lprintf_nl "error in get_url"; 
     raise Not_found
@@ -269,18 +280,19 @@ let rec get_page r content_handler f ferr =
     in
     if !verbose then print_headers ();
     match ans_code with
-      200 ->
-        TcpBufferedSocket.set_closer sock
-            (fun _ _ -> 
-              (* lprintf "default_headers_handler closer\n"; *)
-              f ()
-            );
-
-        let content_length = ref (-1) in
+    | 200 ->
+        ok := true;
+        let content_length = ref (-1L) in
         List.iter (fun (name, content) ->
-            if String.lowercase name = "content-length" then
-            try content_length := int_of_string content
-            with _ -> lprintf_nl "bad content length [%s]" content;
+            match String.lowercase name with
+            | "content-length" ->
+                (try
+                  content_length := Int64.of_string content
+                with _ ->
+                  lprintf_nl "bad content length [%s]" content)
+            | "content-encoding" ->
+                if String.lowercase content = "gzip" then r.req_gzip <- true
+            | _ -> ()
         ) headers;
         let location = "Location", Url.to_string old_url in
         let content_handler = content_handler !content_length (location::headers) in
@@ -340,7 +352,7 @@ let rec get_page r content_handler f ferr =
     | 404 ->
         lprintf_nl "404: Not found for: %s" (Url.to_string_no_args r.req_url);
         close sock (Closed_for_error "bad reply");
-        ferr ans_code;
+        ferr (`HTTP ans_code);
         raise Not_found
 
     | 502 | 503 | 504 ->
@@ -358,7 +370,7 @@ let rec get_page r content_handler f ferr =
     end
         else begin
           lprintf_nl "more than %d retries, aborting." r.req_max_retry;
-          ferr ans_code;
+          ferr (`HTTP ans_code);
           raise Not_found
         end
           
@@ -366,16 +378,28 @@ let rec get_page r content_handler f ferr =
         lprintf_nl "%d: bad reply for: %s"
           ans_code (Url.to_string_no_args r.req_url);
         close sock (Closed_for_error "bad reply");
-        ferr ans_code;
+        ferr (`HTTP ans_code);
         raise Not_found
   in
   get_url 0 r
-  
+
+(** Copy all data from [input] to [output] *)
+let io_copy input output =
+  try
+    let size = 16 * 1024 in
+    let s = String.create size in
+    while true do
+      let n = IO.input input s 0 size in
+      if n = 0 then raise IO.No_more_input;
+      ignore (IO.really_output output s 0 n)
+    done
+  with IO.No_more_input -> ()
+
 let wget r f = 
-  
+
   let file_buf = Buffer.create 1000 in
-  let file_size = ref 0 in
-  
+  let file_size = ref 0L in
+
   try
   get_page r (fun maxlen headers sock nread ->
 (*      lprintf "received %d\n" nread; *)
@@ -383,13 +407,13 @@ let wget r f =
       
       if nread > 0 then begin
           let left = 
-            if maxlen >= 0 then
-              min (maxlen - !file_size) nread
+            if maxlen >= 0L then
+              min (Int64.to_int (maxlen -- !file_size)) nread
             else nread
           in
           Buffer.add_string file_buf (String.sub buf.buf buf.pos left);
           buf_used buf left;
-          file_size := !file_size + left;
+          file_size := !file_size ++ (Int64.of_int left);
           if nread > left then
             TcpBufferedSocket.close sock Closed_by_user
         end
@@ -417,7 +441,20 @@ let wget r f =
 
       let filename = Filename.concat webinfos_dir base in
       if !verbose then lprintf_nl "Filename: %s" filename;
-      Unix2.tryopen_write_bin filename (fun oc -> output_string oc s);
+      if r.req_gzip then
+      begin
+        try
+          Unix2.tryopen_write_bin filename begin fun oc ->
+            let gz = Gzip.input_io (IO.input_string s) in
+            io_copy gz (IO.output_channel oc)
+          end
+        with e ->
+          lprintf_nl "Exception %s while uncompressing content from %s" (Printexc2.to_string e) (Url.to_string r.req_url);
+          Sys.remove filename;
+          raise Not_found
+      end
+      else
+        Unix2.tryopen_write_bin filename (fun oc -> output_string oc s);
       if r.req_save_to_file_time <> 0. then
         Unix.utimes filename r.req_save_to_file_time r.req_save_to_file_time;
       try
@@ -444,10 +481,10 @@ let whead2 r f ferr =
 
 let whead r f = whead2 r f def_ferr
 
-let wget_string r f progress =
+let wget_string r f ?(ferr=def_ferr) progress =
     
   let file_buf = Buffer.create 1000 in
-  let file_size = ref 0 in
+  let file_size = ref 0L in
 
   get_page r
     (fun maxlen headers sock nread ->
@@ -455,20 +492,31 @@ let wget_string r f progress =
         
         if nread > 0 then begin
             let left = 
-              if maxlen >= 0 then
-                min (maxlen - !file_size) nread
+              if maxlen >= 0L then
+                min (Int64.to_int (maxlen -- !file_size)) nread
               else nread
           in
           Buffer.add_string file_buf (String.sub buf.buf buf.pos left);
           progress left maxlen;
           buf_used buf left;
-          file_size := !file_size + left;
+          file_size := !file_size ++ (Int64.of_int left);
           if nread > left then
             TcpBufferedSocket.close sock Closed_by_user
         end)
-  (fun _ ->  
-      f (Buffer.contents file_buf)
-  ) def_ferr
+  (fun _ ->
+      let content = 
+        if r.req_gzip then
+          try
+            let io = Gzip.input_io (IO.input_string (Buffer.contents file_buf)) in
+            IO.read_all io
+          with e -> 
+            lprintf_nl "Exception %s while uncompressing content from %s" (Printexc2.to_string e) (Url.to_string r.req_url);
+            raise Not_found
+        else
+          Buffer.contents file_buf
+      in
+      f content
+  ) ferr
 
 
 let split_header header =
